// filter_wrap.cpp
// FILTER\̃bp
// Copyright (c) 2008 N099
//
#include <windows.h>
#include <iostream>
using namespace std;

#include "filter_wrap.h"


struct FltFlgTbl {
	int flg;
	const char *name;
};

struct WMFltTbl {
	unsigned int msg;
	const char *name;
};

#define INI_FF(n) { FILTER_FLAG_##n, #n }
#define INI_MF(n) { ##n, #n }

static const FltFlgTbl tblFltFlag[] = 
{
	INI_FF(ACTIVE), 
	{ 2, "UNKNOWN(2)" },
	INI_FF(ALWAYS_ACTIVE),
	INI_FF(CONFIG_POPUP),
	INI_FF(CONFIG_CHECK),
	INI_FF(CONFIG_RADIO),
	INI_FF(EX_DATA),
	INI_FF(PRIORITY_HIGHEST),
	INI_FF(PRIORITY_LOWEST),
	INI_FF(WINDOW_THICKFRAME),
	INI_FF(WINDOW_SIZE),
	INI_FF(DISP_FILTER),
	INI_FF(REDRAW),
	INI_FF(EX_INFORMATION),
	INI_FF(INFORMATION),
	INI_FF(NO_CONFIG),
	INI_FF(AUDIO_FILTER),
	INI_FF(RADIO_BUTTON),
	INI_FF(WINDOW_HSCROLL),
	INI_FF(WINDOW_VSCROLL),
	INI_FF(INTERLACE_FILTER),
	INI_FF(NO_INIT_DATA),
	INI_FF(IMPORT),
	INI_FF(EXPORT),
	INI_FF(MAIN_MESSAGE)
};

static const WMFltTbl tblWMsg[] = 
{
	INI_MF(WM_FILTER_UPDATE),
	INI_MF(WM_FILTER_FILE_OPEN),
	INI_MF(WM_FILTER_FILE_CLOSE), 
	INI_MF(WM_FILTER_INIT),
	INI_MF(WM_FILTER_EXIT),
	INI_MF(WM_FILTER_SAVE_START),
	INI_MF(WM_FILTER_SAVE_END),
	INI_MF(WM_FILTER_IMPORT),
	INI_MF(WM_FILTER_EXPORT), 
	INI_MF(WM_FILTER_CHANGE_ACTIVE),
	INI_MF(WM_FILTER_CHANGE_WINDOW), 
	INI_MF(WM_FILTER_CHANGE_PARAM),
	INI_MF(WM_FILTER_CHANGE_EDIT),
	{ 0, NULL },
	INI_MF(WM_FILTER_MAIN_MOUSE_DOWN),
	INI_MF(WM_FILTER_MAIN_MOUSE_UP), 
	INI_MF(WM_FILTER_MAIN_MOUSE_MOVE),
	INI_MF(WM_FILTER_MAIN_KEY_DOWN),
	INI_MF(WM_FILTER_MAIN_KEY_UP),  
	INI_MF(WM_FILTER_MAIN_MOVESIZE),
	INI_MF(WM_FILTER_MAIN_MOUSE_DBLCLK)
};

static const int MSG_BUF_SIZE = 64;

///ATFilter
//display info
void ATFilter::PrintFlag(ostream& os) const
{
	bool first = true;

	os << "\tFILTER_FLAG: ";
	for(int i=0; i<ARRAYSIZE(tblFltFlag); ++i){
		if((flag & tblFltFlag[i].flg) == 0){
			continue;
		}

		if(first){
			os << tblFltFlag[i].name;
			first = false;
		}else{
			os << "|" << tblFltFlag[i].name;
		}
	}

	if(first){
		os << "NONE" << endl;
	}else{
		os << endl;
	}
}

void ATFilter::PrintWindowSize(ostream& os) const
{
	os << "\twindow size (x,y): ";
	if((flag & FILTER_FLAG_WINDOW_SIZE) == 0){
		os << "(default)" << endl;
	}else{
		const int mask = ~(FILTER_WINDOW_SIZE_CLIENT | FILTER_WINDOW_SIZE_ADD);

		if(x & FILTER_WINDOW_SIZE_CLIENT){
			os << "*" << (x & mask);
		}else if(x & FILTER_WINDOW_SIZE_ADD){
			os << "+" << (x & mask);
		}else{
			os << x;
		}

		if(y & FILTER_WINDOW_SIZE_CLIENT){
			os << ",*" << (y & mask) << endl;
		}else if(x & FILTER_WINDOW_SIZE_ADD){
			os << ",+" << (y & mask) << endl;
		}else{
			os << "," << y << endl;
		}
	}
}

void ATFilter::PrintTrack(ostream& os) const
{
	char buf[MSG_BUF_SIZE];

	sprintf_s(buf, sizeof(buf), "\ttrack: %d (ptr:0x%p)", track_n, track);
	os << buf << endl;

	if(track_n){
		for(int i=0; i<track_n; ++i){
			sprintf_s(buf, sizeof(buf), "\t\t[%02d]:%s(%d,%d,%d)", i,
				track_name ? track_name[i] : "\"null\"",
				track_default ? track_default[i] : 0,
				track_s ? track_s[i] : 0, track_e ? track_e[i] : 0);
			os << buf << endl;
		}
	}
}

void ATFilter::PrintCheck(ostream& os) const
{
	char buf[MSG_BUF_SIZE];

	sprintf_s(buf, sizeof(buf), "\tcheck: %d (ptr:0x%p)", check_n, check);
	os << buf << endl;

	if(check_n){
		for(int i=0; i<check_n; ++i){
			sprintf_s(buf, sizeof(buf), "\t\t[%02d]:%s(%d)", i,
				check_name ? check_name[i] : "\"null\"",
				check_default ? check_default[i] : 0);
			os << buf << endl;
		}
	}
}

void ATFilter::PintExData(ostream& os) const
{
	char buf[MSG_BUF_SIZE];

	sprintf_s(buf, sizeof(buf), "\tex_data: ptr:0x%p, def:0x%p, size:%d",
		ex_data_ptr, ex_data_def, ex_data_size);
	os << buf << endl;
}

void ATFilter::PrintMisc(ostream& os) const
{
	char buf[MSG_BUF_SIZE];

	sprintf_s(buf, sizeof(buf), "\texfunc: 0x%p", exfunc);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\tHWND: 0x%p", hwnd);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\tdll_hinst: 0x%p", dll_hinst);
	os << buf << endl;
	if(dll_path){
		sprintf_s(buf, sizeof(buf), "\tdll_path: %s", dll_path);
		os << buf << endl;
	}
}

void ATFilter::PrintFunc(ostream& os) const
{
	char buf[MSG_BUF_SIZE];

	sprintf_s(buf, sizeof(buf), "\tfunctions:");
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_proc()        : 0x%p", func_proc);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_init()        : 0x%p", func_init);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_exit()        : 0x%p", func_exit);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_update()      : 0x%p", func_update);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_WndProc()     : 0x%p", func_WndProc);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_save_start()  : 0x%p", func_save_start);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_save_end()    : 0x%p", func_save_end);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_is_saveframe(): 0x%p", func_is_saveframe);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_project_load(): 0x%p", func_project_load);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_project_save(): 0x%p", func_project_save);
	os << buf << endl;
	sprintf_s(buf, sizeof(buf), "\t\tfunc_modify_title(): 0x%p", func_modify_title);
	os << buf << endl;
}

void ATFilter::PrintWndMsg(std::ostream& os, UINT msg) const
{
	for(unsigned int i=0; i<ARRAYSIZE(tblWMsg); ++i){
		if(tblWMsg[i].msg == msg){
			os << tblWMsg[i].name << endl;
			return;
		}
	}
	os << "WM_?" << endl;
}

///ATFWrap
void ATFWrap::DispFilterInfo(std::ostream& os) const
{
	os << "\tID: " << _id << " \"" << _oauf.name << "\"" << std::endl;
	_oauf.PrintFlag(os);
	_oauf.PrintWindowSize(os);
	_oauf.PrintTrack(os);
	_oauf.PrintCheck(os);
	_oauf.PintExData(os);
	_oauf.PrintMisc(os);
	_oauf.PrintFunc(os);
}

void ATFWrap::WrapFunc(ATFFuncTable *tbl, EXFUNC *exf)
{
	wrap_fptr(_auf->func_proc, tbl->proc);
	wrap_fptr(_auf->func_init, tbl->init);
	wrap_fptr(_auf->func_exit, tbl->exit);
	wrap_fptr(_auf->func_update, tbl->update);
	wrap_fptr(_auf->func_WndProc, tbl->WndProc);
	wrap_fptr(_auf->func_save_start, tbl->save_start);
	wrap_fptr(_auf->func_save_end, tbl->save_end);
	wrap_fptr(_auf->func_is_saveframe, tbl->is_saveframe);
	wrap_fptr(_auf->func_project_load, tbl->project_load);
	wrap_fptr(_auf->func_project_save, tbl->project_save);
	wrap_fptr(_auf->func_modify_title, tbl->modify_title);
	wrap_fptr(_auf->exfunc, exf);
}

void ATFWrap::Restore()
{
	_auf->func_proc = _oauf.func_proc;
	_auf->func_init = _oauf.func_init;
	_auf->func_exit = _oauf.func_exit;
	_auf->func_update = _oauf.func_update;
	_auf->func_WndProc = _oauf.func_WndProc;
	_auf->func_save_start = _oauf.func_save_start;
	_auf->func_save_end = _oauf.func_save_end;
	_auf->func_is_saveframe = _oauf.func_is_saveframe;
	_auf->func_project_load = _oauf.func_project_load;
	_auf->func_project_save = _oauf.func_project_save;
	_auf->func_modify_title = _oauf.func_modify_title;
	_auf->exfunc = _oauf.exfunc;
}

BOOL ATFWrap::FuncProcOrig(void *fp, FILTER_PROC_INFO *fpip)
{
	return _oauf.func_proc(fp, fpip);
}

BOOL ATFWrap::FuncInitOrig(void *fp)
{
	return _oauf.func_init(fp);
}

BOOL ATFWrap::FuncExitOrig(void *fp)
{
	return _oauf.func_exit(fp);
}

BOOL ATFWrap::FuncUpdateOrig(void *fp, int n)
{
	return _oauf.func_update(fp, n); 
}

BOOL ATFWrap::
FuncWndProcOrig(HWND h, UINT m, WPARAM w, LPARAM l, void *e, void *f)
{ 
	return _oauf.func_WndProc(h, m, w, l, e, f);
}

BOOL ATFWrap::FuncSaveStartOrig(void *fp, int s, int e, void *editp)
{
	return _oauf.func_save_start(fp, s, e, editp);
}

BOOL ATFWrap::FuncSaveEndOrig(void *fp, void *editp)
{
	return _oauf.func_save_end(fp, editp);
}

BOOL ATFWrap::
FuncIsSaveFrameOrig(void *fp, void *editp, int sno, int frame, int fps, int eflg, int ires)
{
	return _oauf.func_is_saveframe(fp, editp, sno, frame, fps, eflg, ires);
}

BOOL ATFWrap::FuncProjectLoadOrig(void *fp, void *editp, void *dat, int size)
{
	return _oauf.func_project_load(fp, editp, dat, size);
}

BOOL ATFWrap::FuncProjectSaveOrig(void *fp, void *editp, void *dat, int *size)
{
	return _oauf.func_project_save(fp, editp, dat, size);
}

BOOL ATFWrap::
FuncModifyTitleOrig(void *fp, void *editp, int frame, LPSTR title, int max)
{
	return _oauf.func_modify_title(fp, editp, frame, title, max);
}

